@@ -22,11 +22,11 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include "aj_target.h"
-#include "aj_net.h"
-#include "aj_status.h"
-#include "aj_util.h"
-#include "aj_auth_listener.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_net.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_auth_listener.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -52,7 +52,7 @@ typedef uint32_t (*AJ_AuthPwdFunc)(uint8_t* buffer, uint32_t bufLen);
 /**
  * Callback function prototype for authentication listener
  *
- * @param authmechansim The authentication mechanism used
+ * @param authmechanism The authentication mechanism used
  * @param command The listener command
  * @param creds The credentials
  *
@@ -60,8 +60,47 @@ typedef uint32_t (*AJ_AuthPwdFunc)(uint8_t* buffer, uint32_t bufLen);
  */
 typedef AJ_Status (*AJ_AuthListenerFunc)(uint32_t authmechanism, uint32_t command, AJ_Credential* creds);
 
+/**
+ * Callback function prototype for handling a factory reset request
+ *
+ * @return
+ *         - AJ_OK if the application was able to reset its own state
+ *         - An error status otherwise
+ */
+typedef AJ_Status (*AJ_FactoryResetFunc)();
+
+/**
+ * Callback function prototype for handling a local policy change notification
+ */
+typedef void (*AJ_PolicyChangedFunc)();
+
 #define AJ_MAX_NAME_SIZE 20  /**< Maximum length for a bus unique name */
 
+/**
+ * Session description.
+ *
+ * This is a linked list, maintained within AJ_BusAttachment. There are actually
+ * three kinds of sessions here:
+ *   - truly ongoing sessions (sessionId != 0)
+ *   - bound session ports (sessionId == 0 && host == TRUE)
+ *   - pending JoinSession calls (sessionId == 0 && host == FALSE)
+ *
+ * Note: we only keep track of the bus name at the other end of the session if
+ * we're dealing with a point-to-point session. It's not too hard to extend
+ * this to a list of other participants for a multipoint session, but we don't
+ * have a good use case for this at the moment, so we don't want to waste
+ * memory on this.
+ */
+typedef struct __AJ_Session {
+    int host;                                  /**< Are we hosting this session? */
+    int multipoint;                            /**< Is this a multipoint session? */
+    uint32_t sessionId;                        /**< The session id */
+    char* otherParticipant;                    /**< Other end of the session, in case of a point-to-point session */
+    uint16_t sessionPort;                      /**< session port */
+    uint32_t joinCallSerial;                   /**< call serial for pending JoinSession call */
+    struct __AJ_Session* next;                 /**< Next element in the linked list */
+} AJ_Session;
+
 /**
  * Type for a bus attachment
  */
@@ -72,9 +111,13 @@ typedef struct _AJ_BusAttachment {
     uint32_t serial;                           /**< Next outgoing message serial number */
     AJ_AuthPwdFunc pwdCallback;                /**< Callback for obtaining passwords */
     AJ_AuthListenerFunc authListenerCallback;  /**< Callback for obtaining passwords */
+    uint32_t suites[AJ_AUTH_SUITES_NUM];       /**< Supported cipher suites */
     uint8_t isAuthenticated;                   /**< Has authentication already occured? */
     uint32_t aboutSerial;                      /**< Serial number for About announcement */
     uint8_t isProbeRequired;                   /**< Are probe requests required for the live transport? */
+    AJ_FactoryResetFunc factoryResetCallback;  /**< Callback for handling a factory reset request */
+    AJ_PolicyChangedFunc policyChangedCallback;/**< Callback for handling a local policy change notification */
+    AJ_Session* sessions;                      /**< Linked list describing all ongoing sessions this bus attachment is involved in */
 } AJ_BusAttachment;
 
 /**
@@ -223,6 +266,15 @@ typedef struct _AJ_SessionOpts {
 AJ_EXPORT
 AJ_Status AJ_BusBindSessionPort(AJ_BusAttachment* bus, uint16_t port, const AJ_SessionOpts* opts, uint8_t flags);
 
+/**
+ * Possible response codes for AJ_BusBindSessionPort
+ */
+#define AJ_BINDSESSIONPORT_REPLY_SUCCESS         1   /**< BindSessionPort reply: Success */
+#define AJ_BINDSESSIONPORT_REPLY_ALREADY_EXISTS  2   /**< BindSessionPort reply: SessionPort already exists */
+#define AJ_BINDSESSIONPORT_REPLY_FAILED          3   /**< BindSessionPort reply: Failed */
+#define AJ_BINDSESSIONPORT_REPLY_INVALID_OPTS    4   /**< BindSessionPort reply: Invalid SessionOpts */
+
+
 /**
  * Make a method call to unbind a session port.
  *
@@ -457,8 +509,27 @@ void AJ_BusSetPasswordCallback(AJ_BusAttachment* bus, AJ_AuthPwdFunc pwdCallback
  * @param bus          The bus attachment struct
  * @param authListenerCallback  The auth listener callback function.
  */
+AJ_EXPORT
 void AJ_BusSetAuthListenerCallback(AJ_BusAttachment* bus, AJ_AuthListenerFunc authListenerCallback);
 
+/**
+ * Set a callback for handling requests to factory reset any application state.
+ *
+ * @param bus                   The bus attachment struct
+ * @param factoryResetCallback  The factory reset callback function.
+ */
+AJ_EXPORT
+void AJ_BusSetFactoryResetCallback(AJ_BusAttachment* bus, AJ_FactoryResetFunc factoryResetCallback);
+
+/**
+ * Set a callback for handling security policy change notifications.
+ *
+ * @param bus                    The bus attachment struct
+ * @param policyChangedCallback  The policy changed callback function.
+ */
+AJ_EXPORT
+void AJ_BusSetPolicyChangedCallback(AJ_BusAttachment* bus, AJ_PolicyChangedFunc policyChangedCallback);
+
 /**
  * Callback function prototype for the function called when an authentication completes or fails.
  *
@@ -474,7 +545,7 @@ typedef void (*AJ_BusAuthPeerCallback)(const void* context, AJ_Status status);
  * Initiate a secure connection to a remote peer authenticating if necessary.
  *
  * @param bus            The bus attachment
- * @param peerBusName    The bus name of the remove peer to secure.
+ * @param peerBusName    The bus name of the remote peer to secure.
  * @param callback       A function to be called when the authentication completes
  * @param cbContext      A caller provided context to pass to the callback function
  *
