@@ -23,7 +23,7 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include "aj_target.h"
+#include <ajtcl/aj_target.h>
 
 #ifdef __cplusplus
 extern "C" {
