@@ -22,10 +22,10 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include "aj_target.h"
-#include "aj_status.h"
-#include "aj_bus.h"
-#include "aj_msg.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_bus.h>
+#include <ajtcl/aj_msg.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -98,6 +98,20 @@ extern "C" {
  * requests.
  */
 
+#define MEMBER_TYPE(c) (((c) >> 4) - 2)
+
+#define SIGNAL     MEMBER_TYPE('!')  /* ((0x21 >> 4) - 2) == 0 */
+#define METHOD     MEMBER_TYPE('?')  /* ((0x3F >> 4) - 2) == 1 */
+#define PROPERTY   MEMBER_TYPE('@')  /* ((0x40 >> 4) - 2) == 2 */
+
+#define WRITE_ONLY   '<' /* 0x3C */
+#define READ_WRITE   '=' /* 0x3D */
+#define READ_ONLY    '>' /* 0x3E */
+
+#define SECURE_TRUE  '$' /* Security is required for an interface that start with a '$' character */
+#define SECURE_OFF   '#' /* Security is OFF, i.e. never required for an interface that starts with a '#' character */
+#define SESSIONLESS  '&' /* Only to be used with a signal member types, indicates that the signal will not require a session and is a sessionless signal */
+
 /**
  * Enmeration type for characterizing interface members
  */
@@ -252,6 +266,15 @@ extern const AJ_Object AJ_ROOT_OBJECT;
 AJ_EXPORT
 AJ_Status AJ_RegisterObjectList(const AJ_Object* objList, uint8_t index);
 
+/**
+ * Add all currently registered object list to the access control list.
+ *
+ * @return  - AJ_OK on success
+ *          - AJ_ERR_RESOURCES otherwise
+ */
+AJ_EXPORT
+AJ_Status AJ_RegisterObjectsACL();
+
 /**
  * Register an object list with a specific index providing also a lookup function for the descriptions of objects in the list.
  * This overrides any existing object list and lookup function registered at the specified index.
