@@ -22,19 +22,19 @@
  *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  ******************************************************************************/
 
-#include "aj_target.h"
-#include "aj_debug.h"
-#include "aj_version.h"
-#include "aj_status.h"
-#include "aj_init.h"
-#include "aj_util.h"
-#include "aj_bus.h"
-#include "aj_msg.h"
-#include "aj_introspect.h"
-#include "aj_std.h"
-#include "aj_connect.h"
-#include "aj_about.h"
-#include "aj_helper.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_version.h>
+#include <ajtcl/aj_status.h>
+#include <ajtcl/aj_init.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_bus.h>
+#include <ajtcl/aj_msg.h>
+#include <ajtcl/aj_introspect.h>
+#include <ajtcl/aj_std.h>
+#include <ajtcl/aj_connect.h>
+#include <ajtcl/aj_about.h>
+#include <ajtcl/aj_helper.h>
 
 /**
  * @}
