@@ -20,11 +20,12 @@
  */
 #define AJ_MODULE HELPER
 
-#include "alljoyn.h"
-#include "aj_helper.h"
-#include "aj_link_timeout.h"
-#include "aj_debug.h"
-#include "aj_config.h"
+#include <ajtcl/alljoyn.h>
+#include <ajtcl/aj_helper.h>
+#include <ajtcl/aj_link_timeout.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_config.h>
+#include <ajtcl/aj_security.h>
 
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
@@ -137,6 +138,12 @@ AJ_Status AJ_RunAllJoynService(AJ_BusAttachment* bus, AllJoynConfiguration* conf
             /* Register a callback for providing bus authentication password */
             AJ_BusSetPasswordCallback(bus, config->password_callback);
 
+            /* Register a callback for handling factory reset requests */
+            AJ_BusSetFactoryResetCallback(bus, config->factory_reset_callback);
+
+            /* Register a callback for handling policy change notifications */
+            AJ_BusSetPolicyChangedCallback(bus, config->policy_changed_callback);
+
             /* Configure timeout for the link to the daemon bus */
             AJ_SetBusLinkTimeout(bus, config->link_timeout);
 
@@ -236,7 +243,7 @@ AJ_Status AJ_RunAllJoynService(AJ_BusAttachment* bus, AllJoynConfiguration* conf
          */
         AJ_CloseMsg(&msg);
 
-        if ((status == AJ_ERR_READ) || (status == AJ_ERR_LINK_DEAD)) {
+        if ((status == AJ_ERR_READ) || (status == AJ_ERR_WRITE) || (status == AJ_ERR_LINK_DEAD)) {
             AJ_InfoPrintf(("AJ_RunAllJoynService(): AJ_Disconnect(): daemon \"%s\"\n", AJ_GetUniqueName(bus)));
             AJ_Disconnect(bus);
             connected = FALSE;
@@ -268,6 +275,8 @@ AJ_Status AJ_StartService(AJ_BusAttachment* bus,
     AJ_Status status;
     AJ_Time timer;
     uint8_t serviceStarted = FALSE;
+    uint32_t disposition;
+    uint16_t retport;
 
     AJ_InfoPrintf(("AJ_StartService(bus=0x%p, daemonName=\"%s\", timeout=%d., connected=%d., port=%d., name=\"%s\", flags=0x%x, opts=0x%p)\n",
                    bus, daemonName, timeout, connected, port, name, flags, opts));
@@ -319,9 +328,21 @@ AJ_Status AJ_StartService(AJ_BusAttachment* bus,
             if (msg.hdr->msgType == AJ_MSG_ERROR) {
                 AJ_ErrPrintf(("AJ_StartService(): AJ_METHOD_BIND_SESSION_PORT: %s\n", msg.error));
                 status = AJ_ERR_FAILURE;
-            } else {
-                AJ_InfoPrintf(("AJ_StartService(): AJ_BusRequestName()\n"));
-                status = AJ_BusRequestName(bus, name, flags);
+                break;
+            }
+            status = AJ_UnmarshalArgs(&msg, "uq", &disposition, &retport);
+            if (AJ_OK != status) {
+                break;
+            }
+            if (retport == port) {
+                if (AJ_BINDSESSIONPORT_REPLY_SUCCESS == disposition) {
+                    AJ_InfoPrintf(("AJ_StartService(): AJ_BusRequestName()\n"));
+                    status = AJ_BusRequestName(bus, name, flags);
+                } else {
+                    status = AJ_ERR_FAILURE;
+                    AJ_InfoPrintf(("AJ_StartService(bus=%p): AJ_METHOD_BIND_SESSION_PORT: disposition %d\n", bus, disposition));
+                    break;
+                }
             }
             break;
 
@@ -355,12 +376,18 @@ AJ_Status AJ_StartService(AJ_BusAttachment* bus,
         AJ_CloseMsg(&msg);
     }
 
-    if (status == AJ_OK) {
-        status = AJ_AboutInit(bus, port);
-    } else {
+    if (AJ_OK != status) {
         AJ_WarnPrintf(("AJ_StartService(): AJ_Disconnect(): status=%s\n", AJ_StatusText(status)));
         AJ_Disconnect(bus);
+        return status;
     }
+    status = AJ_AboutInit(bus, port);
+    if (AJ_OK != status) {
+        AJ_WarnPrintf(("AJ_StartService(): AJ_AboutInit returned status=%s\n", AJ_StatusText(status)));
+        AJ_Disconnect(bus);
+        return status;
+    }
+
     return status;
 }
 
@@ -479,15 +506,17 @@ AJ_Status StartClient(AJ_BusAttachment* bus,
 
     while (!clientStarted && (status == AJ_OK)) {
         AJ_Message msg;
-        status = AJ_UnmarshalMsg(bus, &msg, AJ_UNMARSHAL_TIMEOUT);
+        const uint32_t timeout2 = min(AJ_UNMARSHAL_TIMEOUT, timeout);
+
+        status = AJ_UnmarshalMsg(bus, &msg, timeout2);
         if ((status == AJ_ERR_TIMEOUT) && !found) {
             /*
              * Timeouts are expected until we find a name or service
              */
-            if (timeout < AJ_UNMARSHAL_TIMEOUT) {
+            if (timeout <= timeout2) {
                 return status;
             }
-            timeout -= AJ_UNMARSHAL_TIMEOUT;
+            timeout -= timeout2;
             status = AJ_OK;
             continue;
         }
@@ -611,10 +640,13 @@ AJ_Status StartClient(AJ_BusAttachment* bus,
         }
         AJ_CloseMsg(&msg);
     }
-    if (status != AJ_OK && !connected) {
+
+    if ((AJ_OK != status) && !connected) {
         AJ_WarnPrintf(("AJ_StartClient(): Client disconnecting from bus: status=%s\n", AJ_StatusText(status)));
         AJ_Disconnect(bus);
+        return status;
     }
+
     return status;
 }
 
