@@ -23,16 +23,17 @@
  */
 #define AJ_MODULE INTROSPECT
 
-#include "aj_target.h"
-#include "aj_debug.h"
-#include "aj_introspect.h"
-#include "aj_std.h"
-#include "aj_msg.h"
-#include "aj_msg_priv.h"
-#include "aj_debug.h"
-#include "aj_util.h"
-#include "aj_debug.h"
-#include "aj_config.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_introspect.h>
+#include <ajtcl/aj_std.h>
+#include <ajtcl/aj_msg.h>
+#include <ajtcl/aj_msg_priv.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_config.h>
+#include <ajtcl/aj_authorisation.h>
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
  * (usually in debugger).
@@ -41,6 +42,10 @@
 uint8_t dbgINTROSPECT = 0;
 #endif
 
+#if defined(_WIN32)
+#define strcasecmp _stricmp
+#endif
+
 /**
  * The various object lists
  */
@@ -87,27 +92,12 @@ static AJ_DescriptionLookupFunc descriptionLookups[AJ_MAX_OBJECT_LISTS] = { NULL
 #define IN_ARG     '<'  /* 0x3C */
 #define OUT_ARG    '>'  /* 0x3E */
 
-#define WRITE_ONLY '<'  /* 0x3C */
-#define READ_WRITE '='  /* 0x3D */
-#define READ_ONLY  '>'  /* 0x3E */
-
 #define SEPARATOR  ' '
 
 #define IS_DIRECTION(c) (((c) >= IN_ARG) && ((c) <= OUT_ARG))
 
-#define MEMBER_TYPE(c) (((c) >> 4) - 2)
-
-#define SIGNAL     MEMBER_TYPE('!')  /* ((0x21 >> 4) - 2) == 0 */
-#define METHOD     MEMBER_TYPE('?')  /* ((0x3F >> 4) - 2) == 1 */
-#define PROPERTY   MEMBER_TYPE('@')  /* ((0x40 >> 4) - 2) == 2 */
-
-#define SESSIONLESS  '&' /* Only to be uesd with a signal member types, indicates that the signal will not require a session and is a sessionless signal */
 #define IS_SESSIONLESS(c) ((c) == SESSIONLESS)
 
-#define SECURE_TRUE  '$' /* Security is required for an interface that start with a '$' character */
-#define SECURE_OFF   '#' /* Security is OFF, i.e. never required for an interface that starts with a '#' character */
-
-
 static const char* const MemberOpen[] = {
     "  <signal",
     "  <method",
@@ -291,13 +281,7 @@ static AJ_Status ExpandInterfaces(XMLWriterFunc XMLWriter, void* context, const
                  * Advance so that we do not return a '&' character
                  */
                 member++;
-                if (descLookup != NULL) {
-                    /*
-                     * If we have a descLookup function pointer it implies that we entered the method
-                     * through the org.allseen.Introspect interface
-                     */
-                    isSessionless = TRUE;
-                }
+                isSessionless = TRUE;
             }
             attr = ExpandAttribute(XMLWriter, context, &member, nameAttr, "\"");
             if (memberType == PROPERTY) {
@@ -460,6 +444,44 @@ static uint32_t SecurityApplies(const char* ifc, const AJ_Object* obj)
     return FALSE;
 }
 
+/*
+ * Find the best matching language tag, using the lookup algorithm in
+ * RFC 4647 section 3.4.  This algorithm requires that the "supported"
+ * languages be the least specific they can (e.g., "en" in order to match
+ * both "en" and "en-US" if requested), and the "requested" language be
+ * the most specific it can (e.g., "en-US" in order to match either "en-US"
+ * or "en" if supported).
+ */
+#define MAX_LANG_SIZE 63
+static const char* GetBestLanguage(const char* requested)
+{
+    if ((requested != NULL) && (*requested != 0) && (languageList != NULL)) {
+        char languageToCheck[MAX_LANG_SIZE + 1];
+        strncpy(languageToCheck, requested, MAX_LANG_SIZE);
+        languageToCheck[MAX_LANG_SIZE] = '\0';
+        for (;;) {
+            // Look for a supported language matching the language to check.
+            size_t idx;
+            char* pos;
+            for (idx = 0; languageList[idx] != NULL; idx++) {
+                if (strcasecmp(languageList[idx], languageToCheck) == 0) {
+                    return languageList[idx];
+                }
+            }
+
+            // Drop the last subtag and try again.
+            pos = strrchr(languageToCheck, '-');
+            if (pos == NULL) {
+                break;
+            }
+            *pos = 0;
+        }
+    }
+
+    // No match found, so return the default language.
+    return (languageList != NULL) ? languageList[0] : NULL;
+}
+
 static AJ_Status GenXML(XMLWriterFunc XMLWriter, void* context, const AJ_ObjectIterator* objIter, const AJ_Object* virtualObject, const char* languageTag)
 {
     AJ_Status status = AJ_OK;
@@ -492,6 +514,7 @@ static AJ_Status GenXML(XMLWriterFunc XMLWriter, void* context, const AJ_ObjectI
             if (objIter != NULL) {
                 descLookup = descriptionLookups[objIter->l];
             }
+            languageTag = GetBestLanguage(languageTag);
         }
         /*
          * Generate object's XML
@@ -821,6 +844,7 @@ static AJ_Status ComposeSignature(const char* encoding, char direction, char* si
 
 static AJ_Status MatchProp(const char* member, const char* prop, uint8_t op, const char** sig)
 {
+    AJ_Status status;
     const char* encoding = member;
 
     if (*encoding++ != '@') {
@@ -833,16 +857,30 @@ static AJ_Status MatchProp(const char* member, const char* prop, uint8_t op, con
             return AJ_ERR_NO_MATCH;
         }
     }
-    if ((op == AJ_PROP_GET) && (*encoding == WRITE_ONLY)) {
-        AJ_InfoPrintf(("MatchProp(): AJ_ERR_DISALLOWED\n"));
-        return AJ_ERR_DISALLOWED;
+
+    switch (*encoding) {
+    case WRITE_ONLY:
+        status = (AJ_PROP_SET == op) ? AJ_OK : AJ_ERR_DISALLOWED;
+        break;
+
+    case READ_ONLY:
+        status = (AJ_PROP_GET == op) ? AJ_OK : AJ_ERR_DISALLOWED;
+        break;
+
+    case READ_WRITE:
+        status = AJ_OK;
+        break;
+
+    default:
+        AJ_InfoPrintf(("MatchProp(): AJ_ERR_NO_MATCH - incorrect annotation or substring match\n"));
+        status = AJ_ERR_NO_MATCH;
+        break;
     }
-    if ((op == AJ_PROP_SET) && (*encoding == READ_ONLY)) {
-        AJ_InfoPrintf(("MatchProp(): AJ_ERR_DISALLOWED\n"));
-        return AJ_ERR_DISALLOWED;
+    if (AJ_OK == status) {
+        *sig = ++encoding;
     }
-    *sig = ++encoding;
-    return AJ_OK;
+
+    return status;
 }
 
 static uint32_t MatchMember(const char* encoding, const AJ_Message* msg)
@@ -866,9 +904,9 @@ static uint32_t MatchMember(const char* encoding, const AJ_Message* msg)
     return (*encoding == '\0') || (*encoding == ' ');
 }
 
-static AJ_InterfaceDescription FindInterface(const AJ_InterfaceDescription* interfaces, const char* iface, uint8_t* index)
+static AJ_InterfaceDescription FindInterface(const AJ_InterfaceDescription* interfaces, const char* iface, uint8_t* idx)
 {
-    *index = 0;
+    *idx = 0;
     if (interfaces) {
         while (*interfaces) {
             AJ_InterfaceDescription desc = *interfaces++;
@@ -885,7 +923,7 @@ static AJ_InterfaceDescription FindInterface(const AJ_InterfaceDescription* inte
                     return desc;
                 }
             }
-            *index += 1;
+            *idx += 1;
         }
     }
     return NULL;
@@ -954,7 +992,7 @@ AJ_Status AJ_LookupMessageId(AJ_Message* msg, uint8_t* secure)
 /*
  * Validates an index into a NULL terminated array
  */
-static uint8_t CheckIndex(const void* ptr, uint8_t index, size_t stride)
+static uint8_t CheckIndex(const void* ptr, uint8_t idx, size_t stride)
 {
     if (!ptr) {
         return FALSE;
@@ -965,7 +1003,7 @@ static uint8_t CheckIndex(const void* ptr, uint8_t index, size_t stride)
             return FALSE;
         }
         ptr = (((uint8_t*)ptr) + stride);
-    } while (index--);
+    } while (idx--);
     return TRUE;
 }
 #endif
@@ -1045,6 +1083,16 @@ AJ_Status AJ_MarshalPropertyArgs(AJ_Message* msg, uint32_t propId)
     if (secure) {
         msg->hdr->flags |= AJ_FLAG_ENCRYPTED;
     }
+    if (msg->hdr->flags & AJ_FLAG_ENCRYPTED) {
+        /*
+         * Check outgoing access policy
+         */
+        status = AJ_AccessControlCheckProperty(msg, propId, msg->destination, AJ_ACCESS_OUTGOING);
+        if (AJ_OK != status) {
+            return status;
+        }
+    }
+
     /*
      * Marshal interface name
      */
@@ -1082,7 +1130,7 @@ AJ_Status AJ_InitMessageFromMsgId(AJ_Message* msg, uint32_t msgId, uint8_t msgTy
      * allows up to 255 characters in a signature but that would represent an outgrageously complex
      * message argument list.
      */
-    static char msgSignature[32];
+    static char msgSignature[64];
     AJ_Status status = AJ_OK;
 
 #ifndef NDEBUG
@@ -1261,9 +1309,15 @@ AJ_Status AJ_UnmarshalPropertyArgs(AJ_Message* msg, uint32_t* propId, const char
         /*
          * If the interface is secure check the message must be encrypted
          */
-        if ((status == AJ_OK) && secure && !(msg->hdr->flags & AJ_FLAG_ENCRYPTED)) {
-            status = AJ_ERR_SECURITY;
-            AJ_WarnPrintf(("Security violation accessing property\n"));
+        if ((status == AJ_OK) && secure) {
+            if (!(msg->hdr->flags & AJ_FLAG_ENCRYPTED)) {
+                AJ_WarnPrintf(("Security violation accessing property\n"));
+                return AJ_ERR_SECURITY;
+            }
+            /*
+             * Check incoming access policy
+             */
+            status = AJ_AccessControlCheckProperty(msg, *propId, msg->sender, AJ_ACCESS_INCOMING);
         }
     }
     return status;
@@ -1323,6 +1377,17 @@ AJ_Status AJ_MarshalAllPropertiesArgs(AJ_Message* replyMsg, const char* iface, A
                 continue;
             }
 
+            if (secure) {
+                /*
+                 * Check incoming access policy
+                 */
+                status = AJ_AccessControlCheckProperty(replyMsg, propId, replyMsg->destination, AJ_ACCESS_INCOMING);
+                if (AJ_OK != status) {
+                    /* Skip this property */
+                    continue;
+                }
+            }
+
             status = AJ_MarshalContainer(replyMsg, &dict, AJ_ARG_DICT_ENTRY);
             if (status != AJ_OK) {
                 goto Exit;
@@ -1428,23 +1493,38 @@ void AJ_RegisterObjects(const AJ_Object* localObjects, const AJ_Object* proxyObj
     objectLists[AJ_PRX_ID_FLAG] = proxyObjects;
 }
 
+AJ_Status AJ_RegisterObjectsACL()
+{
+    AJ_Status status;
+
+    int i;
+    for (i = 0; i < AJ_MAX_OBJECT_LISTS; i++) {
+        status = AJ_AuthorisationRegister(objectLists[i], i);
+        if (AJ_OK != status) {
+            AJ_WarnPrintf(("AJ_RegisterObjectsACL(): index(%d) %s\n", i, AJ_StatusText(status)));
+            return status;
+        }
+    }
+    return AJ_OK;
+}
+
 void AJ_RegisterDescriptionLanguages(const char* const* languages) {
     languageList = languages;
 }
 
-AJ_Status AJ_RegisterObjectListWithDescriptions(const AJ_Object* objList, uint8_t index, AJ_DescriptionLookupFunc descLookup)
+AJ_Status AJ_RegisterObjectListWithDescriptions(const AJ_Object* objList, uint8_t idx, AJ_DescriptionLookupFunc descLookup)
 {
-    if (index >= ArraySize(objectLists)) {
+    if (idx >= ArraySize(objectLists)) {
         return AJ_ERR_RANGE;
     }
-    objectLists[index] = objList;
-    descriptionLookups[index] = descLookup;
-    return AJ_OK;
+    objectLists[idx] = objList;
+    descriptionLookups[idx] = descLookup;
+    return AJ_AuthorisationRegister(objList, idx);
 }
 
-AJ_Status AJ_RegisterObjectList(const AJ_Object* objList, uint8_t index)
+AJ_Status AJ_RegisterObjectList(const AJ_Object* objList, uint8_t idx)
 {
-    return AJ_RegisterObjectListWithDescriptions(objList, index, NULL);
+    return AJ_RegisterObjectListWithDescriptions(objList, idx, NULL);
 }
 
 AJ_Status AJ_SetProxyObjectPath(AJ_Object* proxyObjects, uint32_t msgId, const char* objPath)
@@ -1544,6 +1624,7 @@ AJ_Status AJ_SetObjectFlags(const char* objPath, uint8_t setFlags, uint8_t clear
 {
     AJ_Status status = AJ_ERR_NO_MATCH;
     AJ_Object* list = (AJ_Object*)objectLists[AJ_APP_ID_FLAG];
+    uint32_t secure = FALSE;
 
     if (list && objPath) {
         /*
@@ -1555,10 +1636,17 @@ AJ_Status AJ_SetObjectFlags(const char* objPath, uint8_t setFlags, uint8_t clear
                 list->flags |= setFlags;
                 AJ_InfoPrintf(("Setting flags for %s to 0x%02x\n", list->path, list->flags));
                 status = AJ_OK;
+                if (AJ_OBJ_FLAG_SECURE & setFlags) {
+                    secure = TRUE;
+                }
             }
             ++list;
         }
     }
+    if (secure) {
+        /* Object became secure, register with the ACL */
+        status = AJ_AuthorisationRegister(objectLists[AJ_APP_ID_FLAG], AJ_APP_ID_FLAG);
+    }
     return status;
 }
 
