@@ -23,19 +23,22 @@
  */
 #define AJ_MODULE SVCLITE
 
-#ifndef NO_SECURITY
+#ifndef TEST_DISABLE_SECURITY
 #define SECURE_INTERFACE
+#define SECURE_OBJECT
 #endif
 
-#include <aj_target.h>
-#include <aj_link_timeout.h>
-#include <aj_debug.h>
-#include <alljoyn.h>
-#include <aj_cert.h>
-#include <aj_creds.h>
-#include <aj_peer.h>
-#include <aj_auth_listener.h>
-#include <aj_authentication.h>
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_link_timeout.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/alljoyn.h>
+#include <ajtcl/aj_cert.h>
+#include <ajtcl/aj_creds.h>
+#include <ajtcl/aj_peer.h>
+#include <ajtcl/aj_auth_listener.h>
+#include <ajtcl/aj_authentication.h>
+#include <ajtcl/aj_authorisation.h>
+#include <ajtcl/aj_security.h>
 
 /**
  * Turn on per-module debug printing by setting this variable to non-zero value
@@ -97,10 +100,18 @@ static const AJ_InterfaceDescription testInterfaces[] = {
 };
 
 static AJ_Object AppObjects[] = {
+#ifdef SECURE_OBJECT
+    { "/org/alljoyn/alljoyn_test", testInterfaces, AJ_OBJ_FLAG_ANNOUNCED | AJ_OBJ_FLAG_SECURE },
+#else
     { "/org/alljoyn/alljoyn_test", testInterfaces, AJ_OBJ_FLAG_ANNOUNCED },
+#endif
     { NULL }
 };
 
+static AJ_PermissionMember members[] = { { "*", AJ_MEMBER_TYPE_ANY, AJ_ACTION_PROVIDE | AJ_ACTION_OBSERVE, NULL } };
+static AJ_PermissionRule rules[] = { { "/org/alljoyn/alljoyn_test", "org.alljoyn.alljoyn_test", members, NULL } };
+static AJ_Manifest manifest = { rules };
+
 /*
  * Message identifiers for the method calls this application implements
  */
@@ -111,7 +122,6 @@ static AJ_Object AppObjects[] = {
 #define APP_TIME_PING       AJ_APP_MESSAGE_ID(0, 1, 2)
 #define APP_MY_SIGNAL       AJ_APP_MESSAGE_ID(0, 1, 3)
 
-
 /*
  * Property identifiers for the properies this application implements
  */
@@ -128,42 +138,11 @@ static void AppDoWork()
     AJ_InfoPrintf(("do work\n"));
 }
 
-
-static const char PWD[] = "123456";
-
 #if defined(SECURE_INTERFACE) || defined(SECURE_OBJECT)
-
-static uint32_t PasswordCallback(uint8_t* buffer, uint32_t bufLen)
-{
-    memcpy(buffer, PWD, sizeof(PWD));
-    return sizeof(PWD) - 1;
-}
-
-// Copied from alljoyn/alljoyn_core/test/bbservice.cc
-static const char pem_prv[] = {
-    "-----BEGIN EC PRIVATE KEY-----"
-    "MDECAQEEIICSqj3zTadctmGnwyC/SXLioO39pB1MlCbNEX04hjeioAoGCCqGSM49"
-    "AwEH"
-    "-----END EC PRIVATE KEY-----"
-};
-
-static const char pem_x509[] = {
-    "-----BEGIN CERTIFICATE-----"
-    "MIIBWjCCAQGgAwIBAgIHMTAxMDEwMTAKBggqhkjOPQQDAjArMSkwJwYDVQQDDCAw"
-    "ZTE5YWZhNzlhMjliMjMwNDcyMGJkNGY2ZDVlMWIxOTAeFw0xNTAyMjYyMTU1MjVa"
-    "Fw0xNjAyMjYyMTU1MjVaMCsxKTAnBgNVBAMMIDZhYWM5MjQwNDNjYjc5NmQ2ZGIy"
-    "NmRlYmRkMGM5OWJkMFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEP/HbYga30Afm"
-    "0fB6g7KaB5Vr5CDyEkgmlif/PTsgwM2KKCMiAfcfto0+L1N0kvyAUgff6sLtTHU3"
-    "IdHzyBmKP6MQMA4wDAYDVR0TBAUwAwEB/zAKBggqhkjOPQQDAgNHADBEAiAZmNVA"
-    "m/H5EtJl/O9x0P4zt/UdrqiPg+gA+wm0yRY6KgIgetWANAE2otcrsj3ARZTY/aTI"
-    "0GOQizWlQm8mpKaQ3uE="
-    "-----END CERTIFICATE-----"
-};
-
 static const char psk_hint[] = "<anonymous>";
 /*
  * The tests were changed at some point to make the psk longer.
- * If doing backcompatibility testing with previous versions (14.08 or before),
+ * If doing backcompatibility testing with previous versions (14.06 or before),
  * define LITE_TEST_BACKCOMPAT to use the old version of the password.
  */
 #ifndef LITE_TEST_BACKCOMPAT
@@ -171,14 +150,38 @@ static const char psk_char[] = "faaa0af3dd3f1e0379da046a3ab6ca44";
 #else
 static const char psk_char[] = "123456";
 #endif
+
+// Copied from alljoyn/alljoyn_core/unit_test/AuthListenerECDHETest.cc with
+// newlines removed
+static const char pem_prv[] = {
+    "-----BEGIN EC PRIVATE KEY-----"
+    "MDECAQEEICCRJMbxSiWUqj4Zs7jFQRXDJdBRPWX6fIVqE1BaXd08oAoGCCqGSM49"
+    "AwEH"
+    "-----END EC PRIVATE KEY-----"
+};
+
+static const char pem_x509[] = {
+    "-----BEGIN CERTIFICATE-----"
+    "MIIBuDCCAV2gAwIBAgIHMTAxMDEwMTAKBggqhkjOPQQDAjBCMRUwEwYDVQQLDAxv"
+    "cmdhbml6YXRpb24xKTAnBgNVBAMMIDgxM2FkZDFmMWNiOTljZTk2ZmY5MTVmNTVk"
+    "MzQ4MjA2MB4XDTE1MDcyMjIxMDYxNFoXDTE2MDcyMTIxMDYxNFowQjEVMBMGA1UE"
+    "CwwMb3JnYW5pemF0aW9uMSkwJwYDVQQDDCAzOWIxZGNmMjBmZDJlNTNiZGYzMDU3"
+    "NzMzMjBlY2RjMzBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABGJ/9F4xHn3Klw7z"
+    "6LREmHJgzu8yJ4i09b4EWX6a5MgUpQoGKJcjWgYGWb86bzbciMCFpmKzfZ42Hg+k"
+    "BJs2ZWajPjA8MAwGA1UdEwQFMAMBAf8wFQYDVR0lBA4wDAYKKwYBBAGC3nwBATAV"
+    "BgNVHSMEDjAMoAoECELxjRK/fVhaMAoGCCqGSM49BAMCA0kAMEYCIQDixoulcO7S"
+    "df6Iz6lvt2CDy0sjt/bfuYVW3GeMLNK1LAIhALNklms9SP8ZmTkhCKdpC+/fuwn0"
+    "+7RX8CMop11eWCih"
+    "-----END CERTIFICATE-----"
+};
+
 static X509CertificateChain* chain = NULL;
-static ecc_privatekey prv;
-static AJ_Status AuthListenerCallback(uint32_t authmechanism, uint32_t command, AJ_Credential*cred)
+static AJ_Status AuthListenerCallback(uint32_t authmechanism, uint32_t command, AJ_Credential* cred)
 {
     AJ_Status status = AJ_ERR_INVALID;
     X509CertificateChain* node;
 
-    AJ_AlwaysPrintf(("AuthListenerCallback authmechanism %d command %d\n", authmechanism, command));
+    AJ_AlwaysPrintf(("AuthListenerCallback authmechanism %08X command %d\n", authmechanism, command));
 
     switch (authmechanism) {
     case AUTH_SUITE_ECDHE_NULL:
@@ -207,12 +210,8 @@ static AJ_Status AuthListenerCallback(uint32_t authmechanism, uint32_t command,
     case AUTH_SUITE_ECDHE_ECDSA:
         switch (command) {
         case AJ_CRED_PRV_KEY:
-            cred->len = sizeof (ecc_privatekey);
-            status = AJ_DecodePrivateKeyPEM(&prv, pem_prv);
-            if (AJ_OK != status) {
-                return status;
-            }
-            cred->data = (uint8_t*) &prv;
+            AJ_ASSERT(sizeof (AJ_ECCPrivateKey) == cred->len);
+            status = AJ_DecodePrivateKeyPEM((AJ_ECCPrivateKey*) cred->data, pem_prv);
             cred->expiration = keyexpiration;
             break;
 
@@ -220,12 +219,7 @@ static AJ_Status AuthListenerCallback(uint32_t authmechanism, uint32_t command,
             switch (cred->direction) {
             case AJ_CRED_REQUEST:
                 // Free previous certificate chain
-                while (chain) {
-                    node = chain;
-                    chain = chain->next;
-                    AJ_Free(node->certificate.der.data);
-                    AJ_Free(node);
-                }
+                AJ_X509FreeDecodedCertificateChain(chain);
                 chain = AJ_X509DecodeCertificateChainPEM(pem_x509);
                 if (NULL == chain) {
                     return AJ_ERR_INVALID;
@@ -323,101 +317,95 @@ static AJ_Status AboutPropGetter(AJ_Message* reply, const char* language)
     char machineIdValue[UUID_LENGTH * 2 + 1];
     machineIdValue[UUID_LENGTH * 2] = '\0';
 
-    if ((language != NULL) && (0 != strcmp(language, "en")) && (0 != strcmp(language, ""))) {
-        /* the language supplied was not supported */
-        status = AJ_ERR_NO_MATCH;
-    }
+    /* Here, "en" is the only supported language, so we always return it
+     * regardless of what was requested, per the algorithm specified in
+     * RFC 4647 section 3.4.
+     */
 
+    status = AJ_MarshalContainer(reply, &array, AJ_ARG_ARRAY);
     if (status == AJ_OK) {
-        status = AJ_MarshalContainer(reply, &array, AJ_ARG_ARRAY);
+        status = AJ_GetLocalGUID(&theAJ_GUID);
         if (status == AJ_OK) {
-            status = AJ_GetLocalGUID(&theAJ_GUID);
+            AJ_GUID_ToString(&theAJ_GUID, machineIdValue, UUID_LENGTH * 2 + 1);
+        }
+        if (status == AJ_OK) {
+            status = MarshalAppId(reply, &machineIdValue[0]);
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "AppName", "s", "svclite");
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "DeviceId", "s", machineIdValue);
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "DeviceName", "s", "Tester");
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "Manufacturer", "s", "QCE");
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "ModelNumber", "s", "1.0");
+        }
+        //SupportedLanguages
+        if (status == AJ_OK) {
+            AJ_Arg dict;
+            AJ_Arg languageListArray;
+            status = AJ_MarshalContainer(reply, &dict, AJ_ARG_DICT_ENTRY);
             if (status == AJ_OK) {
-                AJ_GUID_ToString(&theAJ_GUID, machineIdValue, UUID_LENGTH * 2 + 1);
+                status = AJ_MarshalArgs(reply, "s", "SupportedLanguages");
             }
             if (status == AJ_OK) {
-                status = MarshalAppId(reply, &machineIdValue[0]);
+                status = AJ_MarshalVariant(reply, "as");
             }
             if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "AppName", "s", "svclite");
+                status = AJ_MarshalContainer(reply, &languageListArray, AJ_ARG_ARRAY);
             }
             if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "DeviceId", "s", machineIdValue);
+                status = AJ_MarshalArgs(reply, "s", "en");
             }
             if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "DeviceName", "s", "Tester");
+                status = AJ_MarshalCloseContainer(reply, &languageListArray);
             }
             if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "Manufacturer", "s", "QCE");
+                status = AJ_MarshalCloseContainer(reply, &dict);
             }
-            if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "ModelNumber", "s", "1.0");
-            }
-            //SupportedLanguages
-            if (status == AJ_OK) {
-                AJ_Arg dict;
-                AJ_Arg languageListArray;
-                status = AJ_MarshalContainer(reply, &dict, AJ_ARG_DICT_ENTRY);
-                if (status == AJ_OK) {
-                    status = AJ_MarshalArgs(reply, "s", "SupportedLanguages");
-                }
-                if (status == AJ_OK) {
-                    status = AJ_MarshalVariant(reply, "as");
-                }
-                if (status == AJ_OK) {
-                    status = AJ_MarshalContainer(reply, &languageListArray, AJ_ARG_ARRAY);
-                }
-                if (status == AJ_OK) {
-                    status = AJ_MarshalArgs(reply, "s", "en");
-                }
-                if (status == AJ_OK) {
-                    status = AJ_MarshalCloseContainer(reply, &languageListArray);
-                }
-                if (status == AJ_OK) {
-                    status = AJ_MarshalCloseContainer(reply, &dict);
-                }
 
-            }
-            if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "Description", "s", "svclite test app");
-            }
-            if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "DefaultLanguage", "s", "en");
-            }
-            if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "SoftwareVersion", "s", AJ_GetVersion());
-            }
-            if (status == AJ_OK) {
-                status = AJ_MarshalArgs(reply, "{sv}", "AJSoftwareVersion", "s", AJ_GetVersion());
-            }
         }
         if (status == AJ_OK) {
-            status = AJ_MarshalCloseContainer(reply, &array);
+            status = AJ_MarshalArgs(reply, "{sv}", "Description", "s", "svclite test app");
         }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "DefaultLanguage", "s", "en");
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "SoftwareVersion", "s", AJ_GetVersion());
+        }
+        if (status == AJ_OK) {
+            status = AJ_MarshalArgs(reply, "{sv}", "AJSoftwareVersion", "s", AJ_GetVersion());
+        }
+    }
+    if (status == AJ_OK) {
+        status = AJ_MarshalCloseContainer(reply, &array);
     }
     return status;
 }
 
-uint32_t MyBusAuthPwdCB(uint8_t* buf, uint32_t bufLen)
-{
-    const char* myPwd = "1234";
-    strncpy((char*)buf, myPwd, bufLen);
-    return (uint32_t)strlen(myPwd);
-}
-
-static const uint32_t suites[3] = { AUTH_SUITE_ECDHE_ECDSA, AUTH_SUITE_ECDHE_PSK, AUTH_SUITE_ECDHE_NULL };
-static const size_t numsuites = 3;
+static const uint32_t suites[] = { AUTH_SUITE_ECDHE_ECDSA, AUTH_SUITE_ECDHE_PSK, AUTH_SUITE_ECDHE_NULL };
 
 #define CONNECT_TIMEOUT    (1000 * 1000)
 #define UNMARSHAL_TIMEOUT  (1000 * 5)
 
+#ifdef MAIN_ALLOWS_ARGS
+int AJ_Main(int ac, char** av)
+#else
 int AJ_Main()
+#endif
 {
     AJ_Status status = AJ_OK;
     AJ_BusAttachment bus;
     uint8_t connected = FALSE;
     uint32_t sessionId = 0;
-    X509CertificateChain* node;
+    uint8_t claim = FALSE;
 
     /*
      * One time initialization before calling any other AllJoyn APIs
@@ -428,7 +416,18 @@ int AJ_Main()
     AJ_RegisterObjects(AppObjects, NULL);
     AJ_AboutRegisterPropStoreGetter(AboutPropGetter);
 
-    SetBusAuthPwdCallback(MyBusAuthPwdCB);
+#ifdef MAIN_ALLOWS_ARGS
+    ac--;
+    av++;
+    if (ac) {
+#if defined(SECURE_INTERFACE) || defined(SECURE_OBJECT)
+        if (0 == strncmp(*av, "-claim", 6)) {
+            claim = TRUE;
+        }
+#endif
+    }
+#endif
+
     while (TRUE) {
         AJ_Message msg;
 
@@ -453,10 +452,12 @@ int AJ_Main()
 
 #if defined(SECURE_INTERFACE) || defined(SECURE_OBJECT)
 
-            /* Register a callback for providing bus authentication password */
-            AJ_BusSetPasswordCallback(&bus, PasswordCallback);
-            AJ_BusEnableSecurity(&bus, suites, numsuites);
+            AJ_BusEnableSecurity(&bus, suites, ArraySize(suites));
             AJ_BusSetAuthListenerCallback(&bus, AuthListenerCallback);
+            AJ_ManifestTemplateSet(&manifest);
+            if (claim) {
+                AJ_SecuritySetClaimConfig(&bus, APP_STATE_CLAIMABLE, CLAIM_CAPABILITY_ECDHE_PSK, 0);
+            }
 #endif
 
             /* Configure timeout for the link to the daemon bus */
@@ -495,8 +496,11 @@ int AJ_Main()
                         status = AJ_BusReplyAcceptSession(&msg, TRUE);
                         AJ_InfoPrintf(("Accepted session session_id=%u joiner=%s\n", sessionId, joiner));
                     } else {
-                        status = AJ_BusReplyAcceptSession(&msg, FALSE);
-                        AJ_InfoPrintf(("Accepted rejected session_id=%u joiner=%s\n", sessionId, joiner));
+                        status = AJ_ResetArgs(&msg);
+                        if (AJ_OK != status) {
+                            break;
+                        }
+                        status = AJ_BusHandleBusMessage(&msg);
                     }
                 }
                 break;
@@ -598,23 +602,20 @@ int AJ_Main()
     }
     AJ_WarnPrintf(("svclite EXIT %d\n", status));
 
-#if defined(SECURE_INTERFACE) || defined(SECURE_OBJECT)
-    // Clean up certificate chain
-    while (chain) {
-        node = chain;
-        chain = chain->next;
-        AJ_Free(node->certificate.der.data);
-        AJ_Free(node);
-    }
-#endif
-
     return status;
 }
 
 
 #ifdef AJ_MAIN
+#ifdef MAIN_ALLOWS_ARGS
+int main(int ac, char** av)
+{
+    return AJ_Main(ac, av);
+}
+#else
 int main()
 {
     return AJ_Main();
 }
 #endif
+#endif
