@@ -20,11 +20,11 @@
 #include <gtest/gtest.h>
 
 extern "C" {
-#include "alljoyn.h"
-#include "aj_util.h"
-#include "aj_debug.h"
-#include "aj_bufio.h"
-#include "aj_crypto.h"
+#include <ajtcl/alljoyn.h>
+#include <ajtcl/aj_util.h>
+#include <ajtcl/aj_debug.h>
+#include <ajtcl/aj_bufio.h>
+#include <ajtcl/aj_crypto.h>
 
 #ifndef NDEBUG
 extern AJ_MutterHook MutterHook;
@@ -91,7 +91,7 @@ static const char* testSignature[] = {
     "ya{ss}",
     "yyyyya{ys}"
 };
-
+#ifndef NDEBUG
 static AJ_Status MsgInit(AJ_Message* msg, uint32_t msgId, uint8_t msgType)
 {
     msg->objPath = "/test/mutter";
@@ -101,6 +101,7 @@ static AJ_Status MsgInit(AJ_Message* msg, uint32_t msgId, uint8_t msgType)
     msg->signature = testSignature[msgId];
     return AJ_OK;
 }
+#endif
 
 static const char* const Fruits[] = {
     "apple", "banana", "cherry", "durian", "elderberry", "fig", "grape"
@@ -154,11 +155,15 @@ class MutterTest : public testing::Test {
         const size_t lengthOfShortGUID = 16;
         strncpy(testBus.uniqueName, "DummyNaaaame.N1", lengthOfShortGUID);
 
+#ifndef NDEBUG
         MutterHook = MsgInit;
+#endif
     }
 
     virtual void TearDown() {
+#ifndef NDEBUG
         MutterHook = NULL;
+#endif
     }
 };
 
