@@ -18,9 +18,10 @@
 
 #include <gtest/gtest.h>
 extern "C" {
-#include <aj_target.h>
-#include "aj_debug.h"
+#include <ajtcl/aj_target.h>
+#include <ajtcl/aj_debug.h>
 }
+
 /** Main entry point */
 int main(int argc, char**argv, char**envArg)
 {
